package de.infoskript.binedit;

import java.awt.Font;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

public class BinEdit extends javax.swing.JFrame {

	boolean fileSelected;

	public BinEdit() {
		fileSelected = false;
		initComponents();
	}

	private void loadFile() {
		if (fileSelected) {
			try {
				byte[] bytes = readSmallBinaryFile(dateinamelabel.getText());
				textarea.setText(gib01Folge(bytes));
			} catch (IOException ex) {
				JOptionPane.showMessageDialog(this, "Fehler beim Lesen der Datei");
			}
		}
	}

	private void saveFile() {
		if (fileSelected) {
			String nullEinsFolge = gib01Folge(textarea.getText());
			try {
				writeSmallBinaryFile(getByteByString(nullEinsFolge), dateinamelabel.getText());
			} catch (ByteNotFullException ex) {
				JOptionPane.showMessageDialog(this, "01-Folge ergibt kein ganzes Byte");
			} catch (IOException ex) {
				JOptionPane.showMessageDialog(this, "Fehler beim Schreiben");
			}
		}
	}

	private byte[] getByteByString(String binaryString) throws ByteNotFullException {

		if (binaryString.length() % 8 == 0) {
			int index = 0;
			int position = 0;

			byte[] bytes = new byte[binaryString.length() / 8];

			while (index < binaryString.length()) {
				String byteString = binaryString.substring(index, index + 8);
				Integer byteAsInt = Integer.parseInt(byteString, 2);
				bytes[position] = byteAsInt.byteValue();
				index += 8;
				position++;
			}
			return bytes;
		} else {
			throw new ByteNotFullException();
		}
	}

	private byte[] readSmallBinaryFile(String aFileName) throws IOException {
		Path path = Paths.get(aFileName);
		return Files.readAllBytes(path);
	}

	private void writeSmallBinaryFile(byte[] aBytes, String aFileName) throws IOException {
		Path path = Paths.get(aFileName);
		Files.write(path, aBytes); //creates, overwrites
	}

	private String gib01Folge(String s) {
		// Ersetzte alles außer 01 mit leerer Zeichenfolge
		return s.replaceAll("[^01]", "");
	}

	private String gib01Folge(byte[] bytes) {
		StringBuilder sb = new StringBuilder();
		int zaehler = 0;
		for (byte b : bytes) {
			zaehler++;
			String byteString = Integer.toBinaryString((b & 0xFF) + 0x100).substring(1);
			if (halbbyteTrennerCheckbox.isSelected()) {
				byteString = byteString.substring(0, 4) + " " + byteString.substring(4, 8);
			}
			sb.append(byteString);
			if (zaehler % Integer.parseInt(byteProZeileInputField.getText()) == 0) {
				sb.append("\n");
			} else {
				sb.append(trennzeichenTextField.getText());
			}
		}
		return sb.toString();
	}

	@SuppressWarnings("unchecked")
        // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
        private void initComponents() {

                scrollPanel = new javax.swing.JScrollPane();
                textarea = new javax.swing.JTextArea();
                jPanel1 = new javax.swing.JPanel();
                openButton = new javax.swing.JButton();
                reloadButton = new javax.swing.JButton();
                halbbyteTrennerCheckbox = new javax.swing.JCheckBox();
                byteProZeileInputField = new javax.swing.JTextField();
                bytesProZeileLabel = new javax.swing.JLabel();
                trennzeichenTextField = new javax.swing.JTextField();
                trennzeichenLabel = new javax.swing.JLabel();
                fontSizeSpinner = new javax.swing.JSpinner();
                jLabel2 = new javax.swing.JLabel();
                jPanel2 = new javax.swing.JPanel();
                jPanel3 = new javax.swing.JPanel();
                saveButton = new javax.swing.JButton();
                saveAsButton = new javax.swing.JButton();
                jLabel1 = new javax.swing.JLabel();
                dateinamelabel = new javax.swing.JLabel();

                FormListener formListener = new FormListener();

                setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
                setMinimumSize(new java.awt.Dimension(300, 0));

                textarea.setColumns(20);
                textarea.setFont(new java.awt.Font("Consolas", 0, 24)); // NOI18N
                textarea.setRows(5);
                scrollPanel.setViewportView(textarea);

                jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder(""));

                openButton.setText("Öffnen");
                openButton.setMaximumSize(null);
                openButton.addActionListener(formListener);

                reloadButton.setText("Neu laden");
                reloadButton.addActionListener(formListener);

                halbbyteTrennerCheckbox.setText("4-Bit-Gruppen");
                halbbyteTrennerCheckbox.addActionListener(formListener);

                byteProZeileInputField.setText("1");
                byteProZeileInputField.addActionListener(formListener);

                bytesProZeileLabel.setText("Byte(s) pro Zeile:");

                trennzeichenTextField.setText("|");
                trennzeichenTextField.addActionListener(formListener);

                trennzeichenLabel.setText("Bytetrenner:");

                fontSizeSpinner.setValue(24);
                fontSizeSpinner.addChangeListener(formListener);

                jLabel2.setText("Schriftgröße:");

                javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
                jPanel1.setLayout(jPanel1Layout);
                jPanel1Layout.setHorizontalGroup(
                        jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(jPanel1Layout.createSequentialGroup()
                                .addContainerGap()
                                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addComponent(openButton, javax.swing.GroupLayout.PREFERRED_SIZE, 190, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addGroup(jPanel1Layout.createSequentialGroup()
                                                .addComponent(halbbyteTrennerCheckbox)
                                                .addGap(18, 18, 18)
                                                .addComponent(bytesProZeileLabel)
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addComponent(byteProZeileInputField, javax.swing.GroupLayout.PREFERRED_SIZE, 42, javax.swing.GroupLayout.PREFERRED_SIZE)))
                                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addGroup(jPanel1Layout.createSequentialGroup()
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                                .addComponent(reloadButton, javax.swing.GroupLayout.PREFERRED_SIZE, 190, javax.swing.GroupLayout.PREFERRED_SIZE))
                                        .addGroup(jPanel1Layout.createSequentialGroup()
                                                .addGap(18, 18, 18)
                                                .addComponent(trennzeichenLabel)
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addComponent(trennzeichenTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 42, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addGap(18, 18, 18)
                                                .addComponent(jLabel2)
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addComponent(fontSizeSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, 57, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addGap(0, 0, Short.MAX_VALUE)))
                                .addContainerGap())
                );
                jPanel1Layout.setVerticalGroup(
                        jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(jPanel1Layout.createSequentialGroup()
                                .addContainerGap()
                                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                        .addComponent(openButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addComponent(reloadButton))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                        .addComponent(byteProZeileInputField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addComponent(bytesProZeileLabel)
                                        .addComponent(trennzeichenTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addComponent(halbbyteTrennerCheckbox)
                                        .addComponent(fontSizeSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addComponent(jLabel2)
                                        .addComponent(trennzeichenLabel))
                                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                );

                javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
                jPanel2.setLayout(jPanel2Layout);
                jPanel2Layout.setHorizontalGroup(
                        jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGap(0, 0, Short.MAX_VALUE)
                );
                jPanel2Layout.setVerticalGroup(
                        jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGap(0, 28, Short.MAX_VALUE)
                );

                jPanel3.setBorder(javax.swing.BorderFactory.createTitledBorder(""));

                saveButton.setText("Speichern");
                saveButton.addActionListener(formListener);

                saveAsButton.setText("Speichern unter...");
                saveAsButton.addActionListener(formListener);

                javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
                jPanel3.setLayout(jPanel3Layout);
                jPanel3Layout.setHorizontalGroup(
                        jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(jPanel3Layout.createSequentialGroup()
                                .addContainerGap()
                                .addComponent(saveButton, javax.swing.GroupLayout.PREFERRED_SIZE, 190, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 218, Short.MAX_VALUE)
                                .addComponent(saveAsButton, javax.swing.GroupLayout.PREFERRED_SIZE, 190, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addContainerGap())
                );
                jPanel3Layout.setVerticalGroup(
                        jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(jPanel3Layout.createSequentialGroup()
                                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                        .addComponent(saveButton)
                                        .addComponent(saveAsButton)))
                );

                jLabel1.setText("Datei:");

                dateinamelabel.setText("Noch keine Datei gewählt");

                javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
                getContentPane().setLayout(layout);
                layout.setHorizontalGroup(
                        layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(layout.createSequentialGroup()
                                .addContainerGap()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addComponent(scrollPanel)
                                        .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                        .addComponent(jPanel3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                                        .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                        .addGroup(layout.createSequentialGroup()
                                                .addComponent(jLabel1)
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addComponent(dateinamelabel)
                                                .addGap(0, 0, Short.MAX_VALUE)))
                                .addContainerGap())
                );
                layout.setVerticalGroup(
                        layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(layout.createSequentialGroup()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addGroup(layout.createSequentialGroup()
                                                .addGap(98, 98, 98)
                                                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                        .addGroup(layout.createSequentialGroup()
                                                .addContainerGap()
                                                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                        .addComponent(jLabel1)
                                        .addComponent(dateinamelabel))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(scrollPanel, javax.swing.GroupLayout.DEFAULT_SIZE, 236, Short.MAX_VALUE)
                                .addContainerGap())
                );

                pack();
        }

        // Code for dispatching events from components to event handlers.

        private class FormListener implements java.awt.event.ActionListener, javax.swing.event.ChangeListener {
                FormListener() {}
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                        if (evt.getSource() == openButton) {
                                BinEdit.this.openButtonActionPerformed(evt);
                        }
                        else if (evt.getSource() == reloadButton) {
                                BinEdit.this.reloadButtonActionPerformed(evt);
                        }
                        else if (evt.getSource() == halbbyteTrennerCheckbox) {
                                BinEdit.this.halbbyteTrennerCheckboxActionPerformed(evt);
                        }
                        else if (evt.getSource() == byteProZeileInputField) {
                                BinEdit.this.byteProZeileInputFieldActionPerformed(evt);
                        }
                        else if (evt.getSource() == trennzeichenTextField) {
                                BinEdit.this.trennzeichenTextFieldActionPerformed(evt);
                        }
                        else if (evt.getSource() == saveButton) {
                                BinEdit.this.saveButtonActionPerformed(evt);
                        }
                        else if (evt.getSource() == saveAsButton) {
                                BinEdit.this.saveAsButtonActionPerformed(evt);
                        }
                }

                public void stateChanged(javax.swing.event.ChangeEvent evt) {
                        if (evt.getSource() == fontSizeSpinner) {
                                BinEdit.this.fontSizeSpinnerStateChanged(evt);
                        }
                }
        }// </editor-fold>//GEN-END:initComponents

        private void saveButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveButtonActionPerformed
		if (fileSelected) {
			saveFile();
		} else {
			JFileChooser fc = new JFileChooser();
			if (fc.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
				dateinamelabel.setText(fc.getSelectedFile().toString());
				fileSelected = true;
				saveFile();
			}
		}
        }//GEN-LAST:event_saveButtonActionPerformed

        private void openButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_openButtonActionPerformed
		JFileChooser fc = new JFileChooser();
		if (fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
			dateinamelabel.setText(fc.getSelectedFile().toString());
			fileSelected = true;
			loadFile();
		}

        }//GEN-LAST:event_openButtonActionPerformed

        private void reloadButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_reloadButtonActionPerformed
		if (fileSelected) {
			loadFile();
		} else {
			JOptionPane.showMessageDialog(this, "Noch keine Datei geöffnet");
		}
        }//GEN-LAST:event_reloadButtonActionPerformed

        private void saveAsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveAsButtonActionPerformed

		JFileChooser fc = new JFileChooser();
		if (fc.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
			dateinamelabel.setText(fc.getSelectedFile().toString());
			fileSelected = true;
			saveFile();
		}
        }//GEN-LAST:event_saveAsButtonActionPerformed

        private void fontSizeSpinnerStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_fontSizeSpinnerStateChanged
		float size = Float.parseFloat(fontSizeSpinner.getModel().getValue().toString());
		Font font = textarea.getFont();
		textarea.setFont(font.deriveFont(size));
        }//GEN-LAST:event_fontSizeSpinnerStateChanged

        private void byteProZeileInputFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_byteProZeileInputFieldActionPerformed
		if (fileSelected) {
			loadFile();
		}
        }//GEN-LAST:event_byteProZeileInputFieldActionPerformed

        private void trennzeichenTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_trennzeichenTextFieldActionPerformed
		if (fileSelected) {
			loadFile();
		}
        }//GEN-LAST:event_trennzeichenTextFieldActionPerformed

        private void halbbyteTrennerCheckboxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_halbbyteTrennerCheckboxActionPerformed
		if (fileSelected) {
			loadFile();
		}
        }//GEN-LAST:event_halbbyteTrennerCheckboxActionPerformed

	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		/* Set the Nimbus look and feel */
		//<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
		 * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
		 */
		try {
			for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
				if ("Nimbus".equals(info.getName())) {
					javax.swing.UIManager.setLookAndFeel(info.getClassName());
					break;
				}
			}
		} catch (ClassNotFoundException ex) {
			java.util.logging.Logger.getLogger(BinEdit.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (InstantiationException ex) {
			java.util.logging.Logger.getLogger(BinEdit.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (IllegalAccessException ex) {
			java.util.logging.Logger.getLogger(BinEdit.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (javax.swing.UnsupportedLookAndFeelException ex) {
			java.util.logging.Logger.getLogger(BinEdit.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
		//</editor-fold>
		//</editor-fold>

		/* Create and display the form */
		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
				new BinEdit().setVisible(true);
			}
		});
	}

        // Variables declaration - do not modify//GEN-BEGIN:variables
        private javax.swing.JTextField byteProZeileInputField;
        private javax.swing.JLabel bytesProZeileLabel;
        private javax.swing.JLabel dateinamelabel;
        private javax.swing.JSpinner fontSizeSpinner;
        private javax.swing.JCheckBox halbbyteTrennerCheckbox;
        private javax.swing.JLabel jLabel1;
        private javax.swing.JLabel jLabel2;
        private javax.swing.JPanel jPanel1;
        private javax.swing.JPanel jPanel2;
        private javax.swing.JPanel jPanel3;
        private javax.swing.JButton openButton;
        private javax.swing.JButton reloadButton;
        private javax.swing.JButton saveAsButton;
        private javax.swing.JButton saveButton;
        private javax.swing.JScrollPane scrollPanel;
        private javax.swing.JTextArea textarea;
        private javax.swing.JLabel trennzeichenLabel;
        private javax.swing.JTextField trennzeichenTextField;
        // End of variables declaration//GEN-END:variables
}
